#!/usr/bin/perl -w  # this path to Perl is your_unix_system-dependent: either /usr/bin/perl or /usr/local/bin/perl
# Author:  Dr. Ruming Li -- December 4, 2010
# Purpose: Check entry ID uniqueness and cross-hybridization of oligo probes
# Usage:   perl oligo-check.pl input_orf_file [input_oligo_file][output_file]
# The default output files are oligo-orf.lst and oligo-stats.txt

use strict;
use warnings;

if ($#ARGV > 3) {print "Too many arguments!\n"; exit;}
if (!defined($ARGV[0])) {$ARGV[0]="input_genes.ffn";}
if (!defined($ARGV[1])) {$ARGV[1]="MOST_tdt";}
if (!defined($ARGV[2])) {$ARGV[2]="oligo-to-gene.lst";}
if (!defined($ARGV[3])) {$ARGV[3]="oligo-stats.txt";}

my ($e,$i,$j,$m,$n,$ok,@idx,@tag,@gid,@acn,@gsb,@loc,@prd,@seq,@oid,@olg,@field);

print "\nThis program will run with input files in the following order:\n
input_orf_seq.ffn_file  = $ARGV[0]
input_oligos.tdt_file   = $ARGV[1]
output_oligo_genes_file = $ARGV[2]
output_oligo_stats_file = $ARGV[3]\n\n";

do {
	print "Proceed to run ? [y/n]: ";
	$_ = <STDIN>;
	if (/^n$/i) {exit;}
} until (/^y$/i);

open(ORFFILE, "<$ARGV[0]") or die "$ARGV[0]: No such a file.\n";
open(OLGFILE, "<$ARGV[1]") or die "$ARGV[1]: No such a file.\n";
open(OUTFILE, ">$ARGV[2]") or die "$!";
open(OUTSTAT, ">$ARGV[3]") or die "$!";

# Read in oligo data
while (<OLGFILE>) {
	next if ($.==1);
	chomp;
	@field = split(/\t/, $_, 4);
	push(@oid,$field[0]);
	push(@olg,$field[2]);
}
close(OLGFILE);

print OUTSTAT "\nNumber of input oligos = ",scalar @olg,"\n";
print "\nNumber of input oligos = ",scalar @olg,"\n";

# Read in orf data by sequence block
local $/="\n>";
while (<ORFFILE>) {
	chomp; # trim the trailing "\n>"
	/>?(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(.+)/;
	push(@gid,$1); # Grab gene ID from header
	#push(@acn,$2); # Grab gene accession
	#push(@gsb,$3); # Grab gene symbol
	#push(@loc,$4); # Grab gene locus/coords
	#push(@prd,$5); # Grab gene product name
	s/.+\n//; # trim the 1st line of sequence block
	s/\n//g;  # strip newlines off sequence block
	push(@seq,$_); # save sequence string
	push(@tag,-1); # array for tagging
}
close(ORFFILE);

print "Number of input orfs = ",scalar @seq,"\n\n";
print OUTSTAT "Number of input orfs = ",scalar @seq,"\n\n";

$/="\n"; # reset to normal input record separator
$e=$#seq;
$m=$#olg;
# Check geneID uniqueness and mark matches
for ($i=0; $i<=$e; $i++) {
	for ($j=0; $j<=$m; $j++) {
		if ($gid[$i] eq $oid[$j]) {
			if ($tag[$i] > -1) {
				print "$oid[$j] not unique in oligo entries!\n";
				print OUTSTAT "$oid[$j] not unique in oligo entries!\n";
				exit;
			}
			$tag[$i]=$j;
			$idx[$j]=$i;
		}
	}
	if ($tag[$i] < 0) {
		print "ORF $gid[$i] has no oligo produced from a commercial software.\n";
		print "This ORF was discarded as there was no oligo constructible.\n";
		print OUTSTAT "ORF $gid[$i] has no oligo produced from a commercial software.\n";
		print OUTSTAT "This ORF was discarded as there was no oligo constructible.\n";
	}
}

# Check if an oligo overlaps all other orfs
print OUTFILE "\nOligo-to-ORF/Gene listing (*** denotes an oligo cross-hybridizing with other ORFs/Genes)\n";
print OUTFILE "========================================================================================\n\n";
$n=$ok=0;
for ($j=0; $j<=$m; $j++) {
	for ($i=0; $i<=$e; $i++) {
		if ($tag[$i]!= $j) {
			if ($seq[$i]=~/$olg[$j]/) {
				print "ORF $oid[$j] has an x-hyb oligo.\n";
				print OUTSTAT "ORF $oid[$j] has an x-hyb oligo.\n";
				print OUTFILE "$oid[$j]***\n$olg[$j]\n$seq[$idx[$j]]\n\n";
				$n += 1;
				last;
			}
		}
	}
	if ($i > $e) {
		print OUTFILE "$oid[$j]\n$olg[$j]\n$seq[$idx[$j]]\n\n";
		$ok += 1;
	}
}

print "\nNumber of x-hyb oligos = $n\n";
print "Number of unique oligos = $ok\n\n";
print OUTSTAT "\nNumber of x-hyb oligos = $n\n";
print OUTSTAT "Number of unique oligos = $ok\n\n";

close(OUTFILE);
close(OUTSTAT);
print "Done!\n\n";
